//============================================================================//
// This source file is part of work done by Reinder Nijhoff (reinder@infi.nl) //
// For the latest info, see http://developer.infi.nl                          //
//                                                                            //
// You're free to use the code in any way you like, modified, unmodified or   //
// cut'n'pasted into your own work.                                           //
//                                                                            //
// Part of this source is based on work by:                                   //
//    - Humus (http://esprit.campus.luth.se/~humus/)                          //
//    - Paul Baker (http://www.paulsprojects.net)                             //
//============================================================================//

#include "material.hpp"
#include "texture.hpp"
#include "image.hpp"
#include "../misc/log.hpp"
#include "../renderer.hpp"
#include "../misc/factory.hpp"
   
//==============================================================================
//  Register to Factory
//==============================================================================
bool fac_material  = Factory_material.reg<Material>( "material" );
  
//==============================================================================
//  Material::Material()
//==============================================================================
Material::Material() {
    clear();  
}

//==============================================================================
//  Material::~Material()
//==============================================================================
Material::~Material() {
}

//==============================================================================
//  Material::clear()
//==============================================================================
void Material::clear() {
	m_values.resize( NUM_OF_TYPES );
	
	set_color( DIFFUSE, white );
	set_color( TRANSPARENCY, white );
	
	set_float( SPECULAR, 0.0f );
	set_float( REFRACTION_INDEX, 0.0f );
	
	m_tex_scale = 1.0f;
}

//==============================================================================
//  Material::init()
//==============================================================================
void Material::init(Data_node* data) {
    std::string s; 
    clear();
    
    Color color;
    float val;
    
    if( data->get_value( "color", color ) ) {
		set_color( DIFFUSE, color );
	}
    if( data->get_value( "reflection", val ) ) {
		set_float( SPECULAR, val );
	}
    if( data->get_value( "refraction_index", val ) ) {
		set_float( REFRACTION_INDEX, val );
	}
    if( data->get_value( "transparency", color ) ) {
		set_color( TRANSPARENCY, color );
	}
	data->get_value( "texture_scale", m_tex_scale );
    
    if( Data_value* value = data->get_value( "texture_color" ) ) {
        set_texture( DIFFUSE, Factory_texture.create( value ) );
    }
    if( Data_value* value = data->get_value( "texture_bumpmap" ) ) {
        set_texture( BUMP, Factory_texture.create( value ) );
    }
    if( Data_value* value = data->get_value( "texture_reflection" ) ) {
        set_texture( SPECULAR, Factory_texture.create( value ) );
    }  
}
  
//==============================================================================
//  Material::get_color()
//==============================================================================
Color Material::get_color( int index, const Vector3f& direction ) {
	const Vector3f &r = direction;
	
	int imax = 0;
	float rmax = fabs(r[0]);
	if ( fabs(r[1]) > rmax ) { rmax = fabs(r[1]); imax = 1; }
	if ( fabs(r[2]) > rmax ) { rmax = fabs(r[2]); imax = 2; }
	
	int face = -1;
	float u = 0.0f, v = 0.0f;
	
	switch ( imax ) {
	case 0: if ( r[0] > 0.0f ) {
                face = 0;
                u = -r[1];	
            } else {
			    face = 1;
                u = r[1];
            }	
		    v = r[2];
		    break;
	case 1: if ( r[1] > 0.0f ) {
                face = 2;
                u = r[0];
            } else {
                face = 3;
                u = -r[0];
            }
            v = r[2];
    		break;
	case 2: u = r[0];
            if ( r[2] > 0.0f ) {
                face = 4;
                v = -r[1];
            } else {
			    face = 5;
                v = r[1];
            }
		    break;
	}
	
	u = 0.5f * (u / rmax + 1.0f);
	v = 0.5f * (v / rmax + 1.0f);
	
	return get_color( index, u, v, face );
}
