//============================================================================//
// This source file is part of work done by Reinder Nijhoff (reinder@infi.nl) //
// For the latest info, see http://developer.infi.nl                          //
//                                                                            //
// You're free to use the code in any way you like, modified, unmodified or   //
// cut'n'pasted into your own work.                                           //
//                                                                            //
// Part of this source is based on work by:                                   //
//    - Humus (http://esprit.campus.luth.se/~humus/)                          //
//    - Paul Baker (http://www.paulsprojects.net)                             //
//============================================================================//

#ifndef  _SHADER_
#define  _SHADER_ 1

#include <vector>
#include <string>
#include <map>

#include "../includes.hpp"
#include "../math/vector.hpp"
#include "../math/matrix.hpp"
#include "../misc/data.hpp"

class Material;
class Primitive;

//==============================================================================
//  Shader_value
//==============================================================================
class Shader_value {
public:
	Shader_value() : m_index(-1) {}
	~Shader_value() {}   
	   
	enum Value_type {
	 	FLOAT,
		VECTOR2F,
		VECTOR3F,
		VECTOR4F,
		MATRIX3X3F,
		MATRIX4X4F,
		MATERIAL_FLOAT,
		MATERIAL_VECTOR2F,
		MATERIAL_VECTOR3F,
		MATERIAL_VECTOR4F,
		MATERIAL_TEXTURE,
		LIGHT_POS,
		LIGHT_INV_RADIUS,
		LIGHT_RADIUS,
		LIGHT_COLOR,
		CAMERA_POS,
		TIME  
    };
    
    Value_type &get_type()             { return m_type; }
    void set_type( Value_type type )   { m_type = type; }
    
    int &get_index()		           { return m_index; }
    void set_index(int index)          { m_index = index; }
	    
    int &get_material_index()          { return m_material_index; }
    void set_material_index(int index) { m_material_index = index; }
    
    float *get_value() 		  		  { return m_value; }	
	void set_value(	float value );
	void set_value(	Vector2f value );
	void set_value(	Vector3f value );
	void set_value(	Vector4f value );	
	
	void set_name( const std::string& name ) { m_name = name; }
	const std::string &get_name() { return m_name; }
private:
	std::string m_name;
	int m_index, m_material_index;
	
	Value_type m_type;
	Vector4f m_value;
};

//==============================================================================
//  Shader
//==============================================================================
class Shader {
public:
	Shader() {}
	virtual ~Shader() {}
	
    virtual void init(Data_node* data);
    virtual void clear();

	void read_from_file(const std::string& name);
	virtual void upload_shader(const std::string &vertex_shader_text, const std::string &fragment_shader_text) {}

	virtual void first_update();
	virtual int bind_value( Shader_value& value ) { return -1; }
	  
	void set_shader_global_constants();  
	void set_shader_constants();
	void set_shader_material_constants(Material *material);
	void set_shader_light_constants(Primitive * light);

	void set_shader_constant(const std::string &name, float constant) 	    { set_shader_constant( m_constants_index[name], constant ); }
	void set_shader_constant(const std::string &name, int constant)         { set_shader_constant( m_constants_index[name], constant ); }
	void set_shader_constant(const std::string &name, Vector2f &constant)   { set_shader_constant( m_constants_index[name], constant ); }
	void set_shader_constant(const std::string &name, Vector3f &constant)   { set_shader_constant( m_constants_index[name], constant ); }
	void set_shader_constant(const std::string &name, Vector4f &constant)   { set_shader_constant( m_constants_index[name], constant ); }
	void set_shader_constant(const std::string &name, Matrix3x3f &constant) { set_shader_constant( m_constants_index[name], constant ); }
	void set_shader_constant(const std::string &name, Matrix4x4f &constant) { set_shader_constant( m_constants_index[name], constant ); }
	
	virtual void set_shader_constant(int index, float constant) {};
	virtual void set_shader_constant(int index, int constant) {};
	virtual void set_shader_constant(int index, Vector2f &constant) {};
	virtual void set_shader_constant(int index, Vector3f &constant) {};
	virtual void set_shader_constant(int index, Vector4f &constant) {};
	virtual void set_shader_constant(int index, Matrix3x3f &constant) {};
	virtual void set_shader_constant(int index, Matrix4x4f &constant) {};
	
	unsigned int get_index() { return m_index; }
private:
	std::map< std::string, int > m_constants_index;
	std::vector< Shader_value *> m_constants, m_material_constants, m_global_constants, m_light_constants, m_shader_constants;		
	
	unsigned int m_index;
};

#endif /* _SHADER_ */

