//============================================================================//
// This source file is part of work done by Reinder Nijhoff (reinder@infi.nl) //
// For the latest info, see http://developer.infi.nl                          //
//                                                                            //
// You're free to use the code in any way you like, modified, unmodified or   //
// cut'n'pasted into your own work.                                           //
//                                                                            //
// Part of this source is based on work by:                                   //
//    - Humus (http://esprit.campus.luth.se/~humus/)                          //
//    - Paul Baker (http://www.paulsprojects.net)                             //
//============================================================================//

#include "log.hpp"
#include "data.hpp"
#include "../math/vector.hpp"
#include "../math/matrix.hpp"
#include <sstream>
#include <iostream>
#include <fstream>

//==============================================================================
//  Data_node::~Data_node()
//==============================================================================
Data_node::~Data_node() {
    for(int i=0, s=m_data.size(); i<s; ++i) {
        delete m_data[i];
    }
}

//==============================================================================
//  Data_node::parse_file()
//==============================================================================
bool Data_node::parse_file(const std::string& name) {
    std::ifstream file( name.c_str() );

    if(!file) {
        log_error << "Unable to open " << name << " in File_input_text::open_file() " << std::endl;
        return false;
    } else {
          bool ret = parse_file( file );   
          file.close();
          
          return ret;
    }
}

//==============================================================================
//  Data_node::parse_file()
//==============================================================================
bool Data_node::parse_file(std::ifstream& file) {
    std::string s;
    while(file >> s) {
        if( s == "/*" ) {
            while( s != "*/") {
                file >> s;
            }    
        } else if( s == "}") {
              return true;   
        } else {
            Data_value* value = new Data_value();
            m_data.push_back( value );
            
            value->set_key( s );            
            
            if(!value->parse_file( file )) {
                return false;                   
            }
        }    
    }
      return true;   
}


Data_node* Data_value::get_node() {
    return m_node;
}

Data_value* Data_node::get_next_value() {
     if( m_index < m_data.size() ) {
	 	  m_index++;
	 	  m_data[ m_index-1 ]->reset();
          return m_data[ m_index-1 ];
     } else {
         return NULL;
     }
}

Data_value* Data_node::get_value(const std::string& key) {
    for(int i=0, s=m_data.size(); i<s; ++i) {
        if( m_data[i]->get_key() == key ) {
            m_data[i]->reset();
            return m_data[i];
        }
    }
    return NULL;
}

Data_node* Data_node::get_node (const std::string& key) {
    if( Data_value* value = get_value( key ) ) {
         return value->get_node(); 
    }
    return NULL;
}
       
bool Data_node::get_value(const std::string& key, std::string& val) {
    if( Data_value* value = get_value( key ) ) {
         return value->read( val ); 
    }
    return false;
}

bool Data_node::get_value(const std::string& key, int& val) {
    if( Data_value* value = get_value( key ) ) {
         return value->read( val ); 
    }
    return false;
}

bool Data_node::get_value(const std::string& key, bool& val) {
    if( Data_value* value = get_value( key ) ) {
         return value->read( val ); 
    }
    return false;
}
bool Data_node::get_value(const std::string& key, float& val) {
    if( Data_value* value = get_value( key ) ) {
         return value->read( val ); 
    }
    return false;
}
bool Data_node::get_value(const std::string& key, Vector4f& val) {
    if( Data_value* value = get_value( key ) ) {
         return value->read( val ); 
    }
    return false;
}
bool Data_node::get_value(const std::string& key, Vector3f& val) {
    if( Data_value* value = get_value( key ) ) {
         return value->read( val ); 
    }
    return false;
}
bool Data_node::get_value(const std::string& key, Vector2f& val) {
    if( Data_value* value = get_value( key ) ) {
         return value->read( val ); 
    }
    return false;
}
bool Data_node::get_value(const std::string& key, Matrix3x3f& val) {
    if( Data_value* value = get_value( key ) ) {
         return value->read( val ); 
    }
    return false;
}    

//==============================================================================
//  Data_value::Data_value()
//==============================================================================
Data_value::Data_value() {
    m_node = NULL;
}

//==============================================================================
//  Data_value::~Data_value()
//==============================================================================
Data_value::~Data_value() {
    if( m_node != NULL ) delete m_node;
}

//==============================================================================
//  Data_value::~parse_file()
//==============================================================================
bool Data_value::parse_file(std::ifstream& file) {
    m_node = NULL;
    std::string s;

    char tmp[256];
    file.getline (tmp, 256);
    std::string line = std::string(tmp);

    std::istringstream iss(line);
        
    while(iss >> s) {
        if( s == "/*" ) {
            while( s != "*/") {
                file >> s;
            }    
        } else if( s == "{" ) {
            m_node = new Data_node();
            m_node->parse_file( file );
            return true;
        } else {
            m_tokens.push_back(s);
        }    
    }
    return true;
}

//==============================================================================
//  File_input_text::next_value()
//==============================================================================
inline bool Data_value::next_value() {
    m_index++;
    if(m_index >= m_tokens.size()) {
        return false;
    } 
    return true;
}

//==============================================================================
//  File_input_text::read()
//==============================================================================
bool Data_value::read(std::string& val) {
    if( !next_value() ) return false;
    
    val = m_tokens[ m_index ];
    return true;
}

//==============================================================================
//  File_input_text::read()
//==============================================================================
bool Data_value::read(int& val) {
    if( !next_value() ) return false;
    
    val = atoi( m_tokens[ m_index ].c_str() );
    return true;
}

//==============================================================================
//  File_input_text::read()
//==============================================================================
bool Data_value::read(bool& val) {
    if( !next_value() ) return false;
    
    std::string s = m_tokens[m_index];
    if( s == "true" || s=="1" || s=="True" || s=="TRUE" ) {
        val = true;
    } else {
        val = false;
    }    
    return true;
}

//==============================================================================
//  File_input_text::read()
//==============================================================================
bool Data_value::read(float& val) {
    if(!next_value()) return false;
    
    val = atof( m_tokens[ m_index ].c_str() );
    return true;
}

//==============================================================================
//  File_input_text::read()
//==============================================================================
bool Data_value::read(Vector4f& val) {
    bool ret = true;
    
    ret &= read(val.x);
    ret &= read(val.y);
    ret &= read(val.z);
    ret &= read(val.w);
    
    return ret;
}

//==============================================================================
//  File_input_text::read()
//==============================================================================
bool Data_value::read(Vector3f& val) {
    bool ret = true;
    
    ret &= read(val.x);
    ret &= read(val.y);
    ret &= read(val.z);
    
    return ret;
}

//==============================================================================
//  File_input_text::read()
//==============================================================================
bool Data_value::read(Vector2f& val) {
    bool ret = true;
    
    ret &= read(val.x);
    ret &= read(val.y);
    
    return ret;
}

//==============================================================================
//  File_input_text::read()
//==============================================================================
bool Data_value::read(Matrix3x3f& val) {
    bool ret = true;
    Vector3f row1, row2, row3;
        
    ret &= read(row1);
    ret &= read(row2);
    ret &= read(row3);
    val.set(row1.x, row1.y, row1.z, row2.x, row2.y, row2.z, row3.x, row3.y, row3.z);
    
    return ret;
}
