//============================================================================//
// This source file is part of work done by Reinder Nijhoff (reinder@infi.nl) //
// For the latest info, see http://developer.infi.nl                          //
//                                                                            //
// You're free to use the code in any way you like, modified, unmodified or   //
// cut'n'pasted into your own work.                                           //
//                                                                            //
// Part of this source is based on work by:                                   //
//    - Humus (http://esprit.campus.luth.se/~humus/)                          //
//    - Paul Baker (http://www.paulsprojects.net)                             //
//============================================================================//

#ifndef  _FILE_DATA_
#define  _FILE_DATA_ 1

#include <string>
#include <vector>

#include "../includes.hpp"

class Vector4f;
class Vector3f;
class Vector2f;
class Matrix3x3f;

class Data_value;

//==============================================================================
//  Data_node
//==============================================================================
class Data_node {
public:
    ~Data_node();
       
    bool parse_file(const std::string& name);
    bool parse_file(std::ifstream& file);
    
    void reset() { m_index = 0; }
    Data_value* get_next_value();

    Data_value* get_value(const std::string& key);
    Data_node* get_node (const std::string& key);
           
    bool get_value(const std::string& key, std::string& val);
    bool get_value(const std::string& key, int& val);
    bool get_value(const std::string& key, bool& val);
    bool get_value(const std::string& key, float& val);
    bool get_value(const std::string& key, Vector4f& val);
    bool get_value(const std::string& key, Vector3f& val);
    bool get_value(const std::string& key, Vector2f& val);
    bool get_value(const std::string& key, Matrix3x3f& val);        
private:
    Data_value* find_value(const std::string& key);    
        
    int m_index;
    std::vector<Data_value *> m_data;
};

//==============================================================================
//  Data_value
//==============================================================================
class Data_value {
public:
    Data_value();
    ~Data_value();  
       
    bool parse_file(std::ifstream& file);   
       
    const std::string& get_key() { return m_key; }
    
    void set_key( const std::string& key ) { m_key = key; }
    void reset() { m_index = -1; }
       
    Data_node* get_node();
           
    bool read(std::string& val);
    bool read(int& val);
    bool read(bool& val);
    bool read(float& val);
    bool read(Vector4f& val);
    bool read(Vector3f& val);
    bool read(Vector2f& val);
    bool read(Matrix3x3f& val);
private:
    bool next_value();
        
    int m_index;
        
    std::string m_key;
    std::vector<std::string> m_tokens;
    Data_node* m_node;
};

#endif /* _FILE_DATA_ */

