//============================================================================//
// This source file is part of work done by Reinder Nijhoff (reinder@infi.nl) //
// For the latest info, see http://developer.infi.nl                          //
//                                                                            //
// You're free to use the code in any way you like, modified, unmodified or   //
// cut'n'pasted into your own work.                                           //
//                                                                            //
// Part of this source is based on work by:                                   //
//    - Humus (http://esprit.campus.luth.se/~humus/)                          //
//    - Paul Baker (http://www.paulsprojects.net)                             //
//============================================================================//

#ifndef  _FACTORY_
#define  _FACTORY_ 1

#include <string>
#include <map>
#include <sstream>

#include "../includes.hpp"

#include "log.hpp"
#include "data.hpp"

//==============================================================================
//  factory_create_object
//==============================================================================
template<typename B, typename T>
B *factory_create_object() {
    return new T();
}

//==============================================================================
//  Factory
//==============================================================================
template<typename B, typename I>
class Factory {	  
protected:
    typedef B *(*create_function)();    
public:
    typedef typename std::map< I, create_function >::iterator Iterator;
    typedef typename std::map< I, B* >::iterator Use_iterator;

    template< typename S >
    bool reg(I unique_id) {
    //    if (m_object_creator.find(unique_id) != m_object_creator.end()) {
    //        return false;
    //    }
        m_object_creator[unique_id] = &factory_create_object< B, S >;

        return true;
    }

    bool unreg(I unique_id) {
        return (m_object_creator.erase( unique_id ) == 1);
    }

    B *get_new(I unique_id) {
        Iterator iter = m_object_creator.find( unique_id );

        if (iter == m_object_creator.end()) {
		   	log_error << "Unable to create class " << unique_id << " in Factory::create()" << std::endl;	 
            return NULL;
        }
        return ((*iter).second)();
    }

    B *create(I unique_id) {        
	  	if( B* tmp = get_new( unique_id ) ) {		
            std::ostringstream oss;
            oss << unique_id << "_" << (int)m_objects.size();
			m_objects[ oss.str() ] = tmp;
  			log_debug_multiple << "name of " << unique_id << ": " << oss.str() << std::endl;   
			return tmp;
        }	
        return NULL;
    }

    B *create(I unique_id, I name) {
	  	if( B* tmp = use( name ) ) {
			log_debug_multiple << name << " already exists in Factory::create()" << std::endl;
    		return tmp;
        }
        if( B *tmp = get_new( unique_id ) ) {
            m_objects[ name ] = tmp;
  			log_debug_multiple << "name of " << unique_id << ": " << name << std::endl;   
    		return tmp;
        }
        return NULL;
    }
    
    B *create(Data_value* value) {
	    std::string action, type;
	    I name;
	    
	    value->reset();
	    if( value->read(action) ) {
            if( action == "create" && value->read(type) ) {
                log_function_multiple << "Create " << type << std::endl;   
				B* tmp;
				if( !value->read(name) ) {
					tmp = create( type );
 				} else {
				  	tmp = create( type, name );   
                }
				if( tmp ) {				  	
				  	tmp->init( value->get_node() );
				  	log_success_multiple << std::endl;
    				return tmp;
				}	
				return NULL;
			} else if( action == "use" && value->read(name) ) {
			  	return use( name );
			} else {
			   log_error << "Unable to create class using action '" << action << "' in Factory::create()" << std::endl;	 
			}  	   
		} else {
            log_error << "Unable to read action in Factory::create()" << std::endl;	 
  		}
  		return NULL;
    }
    
    B *use(I unique_id) {
        Use_iterator iter = m_objects.find(unique_id);

        if (iter == m_objects.end()) {
            return NULL;
        }
        return (*iter).second;
    }
    
    // called from scene.cpp
    void free_data() {
		for(Use_iterator iter = m_objects.begin(); iter != m_objects.end(); iter++) {
			B *tmp = (*iter).second;
		    delete tmp;
		} 
    }
    
    std::map< I, B* > &get_objects() { return m_objects; }
protected:
    std::map< I, create_function > m_object_creator;
	std::map< I, B* > m_objects;	  
};

//==============================================================================
//  get_factory()
//==============================================================================
template<typename B, typename I>
Factory<B,I>& get_factory() {
  static Factory<B,I>* fac = new Factory<B,I>();
  return *fac;
} 

//==============================================================================
//  Factories
//==============================================================================
#define Factory_primitive      get_factory<Primitive, std::string>()
#define Factory_base           get_factory<Base, std::string>()
#define Factory_texture        get_factory<Texture, std::string>()
#define Factory_material       get_factory<Material, std::string>()
#define Factory_image          get_factory<Image, std::string>()
#define Factory_renderer       get_factory<Renderer, std::string>()
#define Factory_camera         get_factory<Camera, std::string>()
#define Factory_controller     get_factory<Controller, std::string>()
#define Factory_background     get_factory<Background, std::string>()
#define Factory_batch          get_factory<Batch, std::string>()
#define Factory_scene          get_factory<Scene, std::string>()
#define Factory_shader         get_factory<Shader, std::string>()		

#endif /* _FACTORY_ */

