//============================================================================//
// This source file is part of work done by Reinder Nijhoff (reinder@infi.nl) //
// For the latest info, see http://developer.infi.nl                          //
//                                                                            //
// You're free to use the code in any way you like, modified, unmodified or   //
// cut'n'pasted into your own work.                                           //
//                                                                            //
// Part of this source is based on work by:                                   //
//    - Humus (http://esprit.campus.luth.se/~humus/)                          //
//    - Paul Baker (http://www.paulsprojects.net)                             //
//============================================================================//

#ifndef _RENDERER_OPENGL_
#define _RENDERER_OPENGL_ 1

#include "../includes.hpp"
#include "../renderer.hpp"
#include "../scene/batch.hpp"
#include "../material/texture.hpp"
#include "../material/shader.hpp"

#include <string>

class Shader;
class Shader_value;

//==============================================================================
//  Shader_opengl
//==============================================================================
class Shader_opengl : public Shader {
public:
	Shader_opengl() {}
	~Shader_opengl() {}

    virtual void init(Data_node* data);
	void upload_shader(const std::string &vertex_shader_text, const std::string &fragment_shader_text);
	
	int bind_value( Shader_value& value );

	void set_shader_constant(int index, float constant);
	void set_shader_constant(int index, int constant);
	void set_shader_constant(int index, Vector2f &constant);
	void set_shader_constant(int index, Vector3f &constant);
	void set_shader_constant(int index, Vector4f &constant);
	void set_shader_constant(int index, Matrix3x3f &constant);
	void set_shader_constant(int index, Matrix4x4f &constant);
private:		
	unsigned int m_vertex_shader, m_fragment_shader;		
};
	  
//==============================================================================
//  Texture_opengl
//==============================================================================
class Texture_opengl : public Texture {
public:
    Texture_opengl() : Texture() {};
    
    virtual void init(Data_node* data);
    
    bool upload_texture();
private:	
	int m_anisotropy;
};

//==============================================================================
//  Batch_opengl
//==============================================================================
class Batch_opengl : public Batch {
public:
    Batch_opengl();    
    
    void render();
    void render_solid( int mask );
	    
    bool upload_batch();
private:
    unsigned int m_vertexbuffer, m_indexbuffer;
};

//==============================================================================
//  Renderer_opengl
//==============================================================================
class Renderer_opengl : public Renderer {
public:
    Renderer_opengl();
    ~Renderer_opengl() {}
    
    // init
    virtual void init(Data_node* data);
    void init();
    void close();
    
    // render
    void render_scene( std::vector<Primitive *> *primitives, std::vector<Primitive *> *lights, Background* background=NULL);
	    
    // viewport
    void set_viewport(const int left, const int top, const int width, const int height);
    void set_renderport(const int left, const int top, const int width, const int height);
    void set_view(Camera* camera);
    void set_view_rotation(Camera* camera);
    void clear_buffer( int mask );
        
	// 2d
	void enable_2d_mode( int width, int height );
	void disable_2d_mode();
	
    // texture
    void bind_texture(int texture_slot, Texture* texture);
    void activate_texture_slot(int texture_slot, int tex_id);
    void disable_texture_slot(int texture_slot);
    void disable_texture_slots();
    
    // shader
    void set_shader(Shader* shader);
 
 	// misc functions	
	void set_cull_face(const int cull_face);
	void set_depth_func(const int depth_func);
	void set_mask(const int mask);
	void set_blending(const int src, const int dest, const int mode = BLEND_ADD);
    void set_scissor(const int x, const int y, const int w, const int h);
    void set_color( const Color& color );
    void set_default();
    
    // render batch
    void render_batch(Batch* batch);    
  
    // debug
    void render_debug_lines();    
private:	
	int	m_left, m_top, m_width, m_height;
	
	std::vector<int> m_active_texture_slots;
};

#endif /* _RENDERER_OPENGL_ */
