//============================================================================//
// This source file is part of work done by Reinder Nijhoff (reinder@infi.nl) //
// For the latest info, see http://developer.infi.nl                          //
//                                                                            //
// You're free to use the code in any way you like, modified, unmodified or   //
// cut'n'pasted into your own work.                                           //
//                                                                            //
// Part of this source is based on work by:                                   //
//    - Humus (http://esprit.campus.luth.se/~humus/)                          //
//    - Paul Baker (http://www.paulsprojects.net)                             //
//============================================================================//

#ifndef _CONTROLLER_
#define _CONTROLLER_ 1

#include "../includes.hpp"
#include "../math/matrix.hpp"
#include "../misc/data.hpp"
#include "../misc/log.hpp"

enum Activity { FROZEN = 0,
                ACTIVE = 1 };

//==============================================================================
//  Controller
//==============================================================================
class Controller {
public:
    Controller() : m_activity( ACTIVE ), m_position(0.0f,0.0f,0.0f) {}
    virtual ~Controller() {}
    
    virtual void update() { m_activity = FROZEN; }
    virtual void init(Data_node* data);
    virtual void first_update() {}
    
    virtual void set_position   (const Vector3f & position)       { m_position = position; };
    virtual void set_orientation(const Orientation & orientation) { m_orientation = orientation; };
    virtual void look_at( const Vector3f & eye, const Vector3f & center, const Vector3f & up) {
        m_orientation.look_at( eye, center, up ); 
    }
    Position    & get_position()     { return m_position;    };
    Orientation & get_orientation()  { return m_orientation; };

    Activity& get_activity_state()   { return m_activity; }
protected:
    Position    m_position;                 
    Orientation m_orientation;
    Activity    m_activity;
};

//==============================================================================
//  Controllable_object
//==============================================================================
class Controllable_object {
public:
    Controllable_object();
    virtual ~Controllable_object();
    
    virtual void update() { m_controller->update(); }
    virtual void init(Data_node* data);
    virtual void first_update();
    
    inline Vector3f& get_position() {
        return m_controller->get_position();
    }
    inline Orientation& get_orientation() {
        return m_controller->get_orientation();
    }
    inline Activity& get_activity_state() {
        return m_controller->get_activity_state();
    }
    void set_controller(Controller* controller) { m_controller = controller; }
    Controller* get_controller() { return m_controller; }
protected:
    Controller *m_controller, *m_default_controller;
};

//==============================================================================
//  Controller_fps
//==============================================================================
class Controller_fps : public Controller {
public:
    Controller_fps();
    
    void init(Data_node* data);
    void first_update();
    
    void set_position (const Vector3f & position) {
        look_at( position, m_center + (position - m_position), m_up);
    }    
    void look_at( const Vector3f & eye, const Vector3f & center, const Vector3f & up) {
        m_position = eye;
        m_up = up;
        m_center = center;
        m_orientation.look_at( eye, center, up );
    }    
    void set_rotation(float angle, Vector3f axis);
    void set_rotation(float angle_x, float angle_z);
    void strafe(float speed);
    void move(float speed);
    
    // update function, check keyboard and mouse input...
    void update(void);
private:
    float m_speed;

    Vector3f m_center;
    Vector3f m_strafe;                      
    Vector3f m_up;
};

//==============================================================================
//  Controller_oscillator
//==============================================================================
class Controller_oscillator : public Controller {
public:
    Controller_oscillator();
    
    void init(Data_node* data);
    void first_update() { m_origin = m_position; }
    
    void update(void);
private:
    Vector3f m_origin, m_amplitude, m_frequency;
};

#endif /* _CONTROLLER_ */

